'****************************************************************************
'**************                LAWSON LABS INC.                 *************
'**************              15 Phoenixville Pike               *************
'**************               Malvern, PA  19355                *************
'**************                 610 725-8800                    *************
'**************                 800 321-5355                    *************
'**************                    LOG140                       *************
'**************            VERSION 1.1     09/6/95              *************
'****************************************************************************

'**************                Program Overview                 *************
'
'               This program makes acquiring data using the Lawson Labs Inc.
'       140 A/D Card easy. It will run on IBM PC, XT, AT & compatible
'       computers with at least 256K bytes of RAM, and 1 floppy drive. It is
'       written for Microsoft QuickBASIC 4.5.

'               First install the 140 A/D card in the computer. See your A/D
'        manual for complete installation instructions.

'               Back up this disk, and store the original in a safe place.
'       The original disk contains 4 files:

'               README.140      : Operating instructions for the program

'               LOG140.EXE      : Executable program

'               LOG140.BAS      : Source Code

'               LOG140.CFG      : Configuration File

'               It is recommended that the program be copied to, and run from
'       the current directory. The program will write a configuration file,
'       "LOG140.CFG", to the current directory. Make sure there is not an
'        unrelated file of the same name in the current directory.

'**************                Programmer Notes                 *************

'       LOG140.EXE was compiled under QuickBASIC 4.5 using the default switch
'       settings. This program will run in stand alone compiled form.
'       LOG140.BAS runs under the QuickBASIC 4.5 environment.

'**************                  Source Code                    *************
 KEY OFF
 CLS : LOCATE 10, 16: PRINT "This program was compiled using QuickBASIC 4.5"
 LOCATE 12, 12: PRINT "QuickBASIC is a copyrighted product of Microsoft Corp."
 LOCATE 14, 29: PRINT "Press space to continue": A$ = INPUT$(1)

 '                        variable arrays for channels 0 to 3
 DIM LABEL$(3)        '  data labels for each A/D channel
 DIM UNIT$(3)         '  data units  for each A/D channel
 DIM DPL%(3)          '  number of decimal places for data display
 DIM OS(3)            '  offset factor for each A/D channel
 DIM SC(3)            '  scaling factor for each A/D channel
 DIM READING(3)       '  reading for each A/D channel number
                     
 E$(0) = "########": E$(1) = "######.#"   'Reading format strings
 E$(2) = "#####.##": E$(3) = "####.###"
 E$(4) = "###.####": E$(5) = "##.#####"

 PF$(0) = "is off.": PF$(1) = "will log data."   'printer messages

 DELAY = 100          ' This is a machine speed-dependent delay count.
                      ' Very fast machines may need a larger count.

 PRTCOMPRESS% = 0           'PRTCOMPRESS = 0 printer not in compressed mode

 EXITERROR$ = "Printer not set to 80 column mode"  'exit program error message

 ON ERROR GOTO ERROROUTINE            'set error trapping
 ERRORSTR$ = "Cannot read configuration file, M140.CFG"
 OPEN "LOG140.CFG" FOR INPUT AS #1    'read configuration file if found
 INPUT #1, ADC, PFLG%, SMPINT, DISK%  'get configuration variables
 FOR N = 0 TO 3
     INPUT #1, LABEL$(N), UNIT$(N), DPL%(N), OS(N), SC(N)
 NEXT N

CLOSECONFIG:
 ERRORSTR$ = "An error has occurred while closing a disk file "
 CLOSE
 IF ADC = 0 THEN GOTO CONFIGURE  'configuration not valid, goto configuration
 START = ADC + 1                 'start conversion address for 140 A/D card
 GOTO MAINMENU

 '*****  MAINMENU  *****
MAINMENU:
 CLS
 LOCATE 6, 35: PRINT "MAIN MENU"
 LOCATE 10, 30: PRINT "1)  GATHER DATA"
 LOCATE 12, 30: PRINT "2)  CONFIGURE SYSTEM"
 LOCATE 14, 30: PRINT "3)  EXIT"
 LOCATE 18, 22: PRINT "Please enter the number of your choice"
 M$ = INPUT$(1): M = VAL(M$)
 IF M < 1 OR M > 3 THEN GOTO MAINMENU
 ON M GOTO GATHER, CONFIGURE
 IF PRTCOMPRESS% = 1 THEN    'if printer logging ON then reset printer & exit
    LOCATE 24, 10: PRINT "Reseting Printer to normal charactor type";
    ERRORSTR$ = EXITERROR$        ' exit program error message
    OPEN "lpt1:" FOR OUTPUT AS #2
    PRINT #2, CHR$(18)            ' reset printer to normal type
    WIDTH #2, 80                  ' reset printer port to 80 columns
 END IF
EXITPROGRAM:
 PRINT : PRINT "done"             ' exit program
END

   '               GATHER, gather data control routine
GATHER:
 CLS
 IF ADC = 0 THEN           'address for 140 A/D card not entered
       LOCATE 10, 17
       PRINT "System not configured. Press space to continue."
       A$ = INPUT$(1): GOTO CONFIGURE      'configuration invalid
 END IF
              ' SCANCONTROL, initialize scanning
 GOSUB OUTSWITCH            'check card before gathering data
SCANCONTROL:
 CLS
 IF DISK% = 1 THEN GOTO SETFILE   'if disk storage on then get data filespec
SETFILERESUME:
 IF PFLG% = 1 THEN                'if printer logging on then set printer to
    CLS : LOCATE 24, 10: PRINT "Printing Header ...."           '132 columns
    ERRORSTR$ = "An error has occurred during printing, check the printer "
    OPEN "LPT1:" FOR OUTPUT AS #2
    PRINT #2, CHR$(15): WIDTH #2, 132: PRINT #2, "DATE", DATE$
    PRINT #2, : PRINT #2, "  SCAN  "; SPC(11); "  TIME  "; SPC(12);
    PRTCOMPRESS% = 1        'PRTCOMPRESS = 1 then printer in compressed mode
    FOR N = 0 TO 3
      IF SC(N) <> 0 THEN
         PRINT #2, USING "\            \"; LABEL$(N);
         IF N <> 3 THEN PRINT #2, SPC(11);
      END IF
    NEXT N
    PRINT #2, CHR$(10)      'print double linefeed
 END IF
 GOSUB FRACDAY: STIME = FOD            'get elapsed Fraction Of Day
 STORAGENUM = 1: DAY = 0               'initilize starting values
 LASTDATE$ = DATE$                     'save for midnight rollover
 GOSUB FRACDAY: NTIME = FOD + SMPINT   'set NTIME to next storage time
 WAIT ADC, 128, 128                    'wait for conversion to finish
 OUT ADC, 0: OUT START, 0              'start channel 0 conversion
 CLS

 '              SCANLOOP, checks storage interval & Esc keypress
SCANLOOP:
 LOCATE 1, 1
 PRINT "DATE: "; DATE$; "    TIME: "; TIME$; "         press Esc for menu"
 GOSUB SCAN            'scan all 4 channels
 A$ = INKEY$
 IF A$ = CHR$(27) THEN GOTO CLOSEFILES   'Esc pressed, close files, Main Menu
 GOSUB FRACDAY
 IF FOD + DAY >= NTIME THEN
    NTIME = NTIME + SMPINT             ' set NTIME to next storage time
    IF DISK% = 1 THEN GOTO SAVEDATA    ' store to disk
SAVEDATARESUME:                        ' entry point after saving data
    IF PFLG% = 1 THEN GOTO PRINTER     ' log on printer
PRINTERESUME:                          ' entry point after printing data
    STORAGENUM = STORAGENUM + 1        ' increment Storage Number
 END IF
 GOTO SCANLOOP


'               SAVEDATA, write Storage Number,Scan Time, & Reading to disk
SAVEDATA:
 ERRORSTR$ = "An error has occurred while writing to disk, check the disk "
 PRINT #1, STORAGENUM; CHR$(34); SCANTIME$; CHR$(34); " ";
 FOR DP = 0 TO 3            'do all 4 channels
                            'if no Scaling Factor, then do not store reading
   IF SC(DP) <> 0 THEN
      PRINT #1, USING E$(DPL%(DP)); READING(DP); : PRINT #1, " ";
   END IF
 NEXT DP
 PRINT #1,
 GOTO SAVEDATARESUME   ' SAVEDATA exit

'               PRINTER,  write data to printer
PRINTER:
 ERRORSTR$ = "An error has occurred during printing, check the printer "
 LOCATE 24, 10: PRINT "Printing ......         ";
 PRINT #2, USING "####"; STORAGENUM; : PRINT #2, SPC(15); SCANTIME$; SPC(9);
 FOR N = 0 TO 3
    IF SC(N) <> 0 THEN
      PRINT #2, USING E$(DPL%(N)); READING(N);
      PRINT #2, " "; : PRINT #2, USING "\     \"; UNIT$(N);
      IF N <> 3 THEN PRINT #2, SPC(9);
    END IF
 NEXT N: PRINT #2, ""
 LOCATE 24, 10: PRINT SPC(20);      '  clear Printing ...... message
 GOTO PRINTERESUME                 '  PRINTER exit
 
'               SETFILE, set the filespec: drive, path, file name,
SETFILE:
 CLS
 LOCATE 10, 20: PRINT "Enter name for data file ";
 INPUT A$: F$ = LEFT$(A$, 30)         'truncate filespec, F$
 IF F$ = "" THEN GOTO SETFILE
 CLS : LOCATE 20, 20: PRINT "Press Esc for main menu."
 LOCATE 10, 20: PRINT "Filename is "; F$; "? (Y/N) "; : A$ = INPUT$(1)
 IF A$ = CHR$(27) THEN GOTO MAINMENU     'Esc pressed then goto Main Menu
 IF A$ <> "y" AND A$ <> "Y" THEN GOTO SETFILE     'loop till valid keystroke
 ERRORSTR$ = "Unable to open     " + F$
 OPEN F$ FOR OUTPUT AS #1
 GOTO SETFILERESUME  '      SETFILE exit


'               SCAN, scan all channels, convert data, display reading
SCAN:
 FOR N = 0 TO 3
 NC = ((N + 1) MOD 4) * 16         'set control code for Next Channel
   GOSUB READCARD                  'get current channel data
   READING = (V + OS(N)) * SC(N)   'convert data
   IF SC(N) <> 0 THEN              'Scaling Factor not = 0, so display reading
     LOCATE ((N + 2) * 2), 2: PRINT USING "\       \"; LABEL$(N);
     PRINT USING E$(DPL%(N)); READING;
     PRINT " ";
     PRINT USING "\     \"; UNIT$(N);
     READING(N) = READING
   END IF
 NEXT N
RETURN   '      SCAN
                                       
'***** CONFIGURE *****, CONFIGURE SYSTEM
CONFIGURE:
 CLS : LOCATE 5, 33: PRINT "CONFIGURATION MENU"
 LOCATE 10, 30: PRINT "1)  DEFINE DATA POINTS"
 LOCATE 12, 30: PRINT "2)  DEFINE DATA RUN"
 LOCATE 14, 30: PRINT "3)  RETURN TO MAIN MENU"
 LOCATE 20, 22: PRINT "Please enter the number of your choice"
 M$ = INPUT$(1): M = VAL(M$)
 IF M < 1 OR M > 3 THEN GOTO CONFIGURE
 ON M GOTO DEFINEDATA, DEFINERUN, SAVECONFIG

'****** DEFINEDATA ******, DEFINE DATA POINTS
DEFINEDATA:
 CLS : GOSUB CHECKSWITCH   'check address of 140 A/D card
 N = 0                     'set to channel 0 variables

CONFIGSCREEN:
 CLS : LOCATE 3, 25: PRINT "CONFIGURATION FOR 140 CHANNEL "; N: PRINT
 NC = N * 16                    'channel selection code for Next Channel
 WAIT ADC, 128, 128             'wait for conversion to end
 OUT ADC, NC: OUT START, 0      'start Next Channel conversion

REWRITE:
 LOCATE 6, 1
 PRINT TAB(10); "1)  LABEL"; TAB(65); LABEL$(N): PRINT
 PRINT TAB(10); "2)  UNITS"; TAB(65); UNIT$(N): PRINT
 PRINT TAB(10); "3)  DECIMAL PLACES TO DISPLAY"; TAB(65); DPL%(N): PRINT
 PRINT TAB(10); "4)  OFFSET FACTOR"; TAB(65); OS(N): PRINT
 PRINT TAB(10); "5)  SCALING FACTOR"; TAB(65); SC(N): PRINT ""
 IF SC(N) <> 0 THEN                'if scaling factor <> 0 then
   GOSUB READCARD                  'read current channel data
   READING = (V + OS(N)) * SC(N)   'convert, & display
   PRINT TAB(7); "FORMATTED READING  ";
   PRINT SPC(10); : PRINT USING E$(DPL%(N)); READING; : PRINT " ";
   PRINT UNIT$(N); "     "
 ELSE
   PRINT TAB(7); "CHANGE SCALING FACTOR TO NON-ZERO TO ";
   PRINT "INCLUDE THIS DATA POINT       "
 END IF: PRINT : PRINT
 PRINT "Enter the number of the item to change, or": PRINT : PRINT
 PRINT "M for menu        N for next one        ";
 PRINT "S for skip        D for delete this one"
 A$ = INKEY$
 IF A$ = "M" OR A$ = "m" THEN GOTO CONFIGURE    'return to configuration menu
 IF A$ = "N" OR A$ = "n" THEN
     N = (N + 1) MOD 4          'select next channel
     GOTO CONFIGSCREEN          'rewrite the screen, start conversion
 END IF
 IF A$ = "S" OR A$ = "s" THEN GOTO SKIPDATA  'select data channel 0-3
 IF A$ = "D" OR A$ = "d" THEN                'clear all channel variables
    LABEL$(N) = "": UNIT$(N) = "": DPL%(N) = 0: OS(N) = 0: SC(N) = 0
    GOTO CONFIGSCREEN                        'rewrite the screen
 END IF
 A = VAL(A$)
 IF A < 1 OR A > 5 THEN GOTO REWRITE         'no valid keystroke
 CLS
 ON A GOSUB SETLABEL, SETUNITS, SETDECIMALPLACES, SETOFFSET, SETSCALE
 GOTO CONFIGSCREEN

SKIPDATA:
 CLS
 LOCATE 10, 13: PRINT "Enter the 140 A/D Converter channel, ";
 PRINT "0 to 3, to display  ";
 A$ = INPUT$(1): N = INT(VAL(A$))
 IF N < 0 OR N > 3 THEN GOTO SKIPDATA   'bad keystroke
 GOTO CONFIGSCREEN

'RETURN     ***** DEFINEDATA *****

'               SETLABEL, set the data label for a channel
SETLABEL:
 LOCATE 10, 15
 PRINT "Enter label for the 140 A/D Converter channel "; N; "  ";
 INPUT A$
 LABEL$(N) = LEFT$(A$, 8)
RETURN    '     SETLABEL

'               SETUNITS, set the reading units for a channel
SETUNITS:
 LOCATE 10, 15
 PRINT "Enter units for the 140 A/D Converter channel "; N; "  ";
 INPUT A$
 UNIT$(N) = LEFT$(A$, 5)
RETURN    '     SETUNITS
            '               SETDECIMALPLACES, set display format for channel
SETDECIMALPLACES:
 LOCATE 10, 8
 PRINT "Enter number of decimal places for the 140 A/D Converter channel ";
 PRINT N; "  ";
 A$ = INPUT$(1)
 DPL%(N) = VAL(A$)
 IF DPL%(N) > 5 THEN DPL%(N) = 5
 IF DPL%(N) < 0 THEN DPL%(N) = 0
RETURN         'SETDECIMALPLACES

'               SETOFFSET, set the offset for a channel
SETOFFSET:
 LOCATE 20, 15: PRINT "Reading = (millivolts + offset) * scaling"
 LOCATE 10, 15
 PRINT "Enter offset for the 140 A/D Converter channel "; N; "  ";
 INPUT A$
 OS(N) = VAL(A$)
 IF OS(N) <> 0 AND (ABS(OS(N)) > 999999! OR ABS(OS(N)) < .000001) THEN
    CLS : GOTO SETOFFSET            ' offset out of range
 END IF
RETURN     '    SETOFFSET

'               SETSCALE, set scaling factor for a channel
SETSCALE:
 LOCATE 20, 15: PRINT "Reading = (millivolts + offset) * scaling"
 LOCATE 10, 15
 PRINT "Enter scaling for the 140 A/D Converter channel "; N; "  ";
 INPUT A$
 SC(N) = VAL(A$)
 IF SC(N) <> 0 AND (ABS(SC(N)) > 999999! OR ABS(SC(N)) < .000001) THEN
    CLS : GOTO SETSCALE             'scaling factor out of range
 END IF
RETURN     '    SETSCALE
'***********   end of DEFINEDATA routines ***********

'***** DEFINERUN *****,  DEFINE DATA RUN
DEFINERUN:
 FOD = SMPINT                  'set Fraction Of Day to sample interval
 HH = INT(FOD * 24)                       'convert to hours,
 MM = INT((FOD - HH / 24) * 1440)                    'minutes,
 SS = CINT((FOD - (HH / 24 + MM / 1440)) * 86400!)   'and seconds
 IF SS = 60 THEN SS = 0: MM = MM + 1: IF MM = 60 THEN HH = HH + 1
 IF SMPINT = 0 THEN GOTO SETINTERVAL

INTERVAL:
 CLS : LOCATE 22, 22: PRINT "Enter C to change or N for next"
 LOCATE 10, 20: PRINT "Sampling interval is "; HH; " hours"; TAB(41);
 PRINT MM; " minutes"; TAB(41); SS; " seconds"
 A$ = INPUT$(1)
 IF A$ = "n" OR A$ = "N" THEN GOTO AUTOSTORE   'set disk spooling on/off
 IF A$ <> "c" AND A$ <> "C" THEN GOTO INTERVAL

SETINTERVAL:
 CLS : LOCATE 10, 21: PRINT "Enter sample interval hours  ";
 INPUT A$: HH = INT(VAL(A$)): IF HH > 24 OR HH < 0 THEN GOTO SETINTERVAL

SETMINUTES:
 LOCATE 12, 41: PRINT "minutes "; SPC(20); : LOCATE 12, 50
 INPUT A$: MM = INT(VAL(A$)): IF MM > 1440 OR MM < 0 THEN GOTO SETMINUTES

SETSECS:
 LOCATE 14, 41: PRINT "seconds "; SPC(20); : LOCATE 14, 50
 INPUT A$: SS = INT(VAL(A$)): IF SS > 86400! OR SS < 0 THEN GOTO SETSECS
 SMPINT = HH / 24 + MM / 1440 + SS / 86400!
 IF SMPINT > 1 THEN
    LOCATE 16, 12
    PRINT "Sample Interval excedes 24 hours, Press Space to continue"
    A$ = INPUT$(1)
    GOTO SETINTERVAL      're-enter sample interval
 END IF
 
 GOTO DEFINERUN

               'AUTOSTORE, turn disk storage on/off
AUTOSTORE:
 CLS : LOCATE 22, 23: PRINT "Enter C to change or N for next"
 LOCATE 12, 27
 IF DISK% = 0 THEN PRINT "Autostore feature is off  ";
 IF DISK% <> 0 THEN PRINT ; "Autostore feature is on  ";
 A$ = INPUT$(1)
 IF A$ = "n" OR A$ = "N" THEN GOTO SETPRINTER    'set printer logging on/off
 IF A$ = "c" OR A$ = "C" THEN
    IF DISK% = 0 THEN DISK% = 1 ELSE DISK% = 0   'toggle disk storage flag
 END IF
 GOTO AUTOSTORE

            '   SETPRINTER, turn printer logging on/off
SETPRINTER:
 LOCATE 12, 27: PRINT "Printer "; PF$(PFLG%); "              ";
 A$ = INPUT$(1)
 IF A$ = "n" OR A$ = "N" THEN GOTO CONFIGURE    'return to configuration menu
 IF A$ = "c" OR A$ = "C" THEN
    IF PFLG% = 0 THEN PFLG% = 1 ELSE PFLG% = 0  'toggle printer logging flag
       
 END IF
 GOTO SETPRINTER
'***********   end of DEFINERUN routines ***********

'*****  SAVECONFIG *****, RETURN TO MAIN MENU
SAVECONFIG:
 CLS : LOCATE 10, 17: PRINT "Do you want to save this configuration? (Y/N)"
 A$ = INPUT$(1)
 IF A$ = "Y" OR A$ = "y" THEN GOTO SAVETODISK
 IF A$ = "N" OR A$ = "n" THEN GOTO MAINMENU
 GOTO SAVECONFIG

'               SAVETODISK, write the configuration file to disk
SAVETODISK:
 ERRORSTR$ = "Unable to save configuration file. "
 OPEN "LOG140.CFG" FOR OUTPUT AS #1
 WRITE #1, ADC, PFLG%, SMPINT, DISK%     'write variables to LOG140.CFG file
 FOR N = 0 TO 3
     WRITE #1, LABEL$(N), UNIT$(N), DPL%(N), OS(N), SC(N)
 NEXT N
 GOTO CLOSEFILES   'Close files, Main Menu

'               CHECKSWITCH, verify address setting on 140 A/D card
CHECKSWITCH:
 IF ADC <> 0 THEN     'see if address for 140 A/D card has changed
   CLS : LOCATE 10, 20: PRINT "Have you changed switch positions? (Y/N)"
   A$ = INPUT$(1)
   IF A$ = "Y" OR A$ = "y" THEN GOTO SWITCHPOSITION
   IF A$ = "n" OR A$ = "N" THEN GOTO OUTSWITCH ELSE GOTO CHECKSWITCH
 END IF
SWITCHPOSITION:
 CLS : LOCATE 10, 5
 PRINT "Please enter the selector switch, 1 to 4, which is in the ON ";
 PRINT "position ":
 LOCATE 12, 20: PRINT "Press Esc to return to Configuration Menu";
 A$ = INPUT$(1)
 IF A$ = CHR$(27) THEN RETURN CONFIGURE
 A = INT(VAL(A$))
 IF A < 1 OR A > 4 THEN GOTO SWITCHPOSITION
 ADC = 804 - A * 8: START = ADC + 1
OUTSWITCH:
 IF INP(ADC) AND 128 THEN GOTO RECHECK
 OUT START, 0: IF INP(ADC) AND 128 THEN RETURN   'A/D card found
RECHECK:
 LOCATE 14, 12
 PRINT "Please recheck switch setting. Press Space to continue"
 A$ = INPUT$(1): GOTO SWITCHPOSITION

           '    READCARD, start conversion for next channel,
           '              and convert data for current channel.
READCARD:
 WAIT ADC, 128, 128    'wait for conversion to finish
 FOR PAUSE = 0 TO DELAY: NEXT PAUSE     'short pause here for fast machines
 OUT ADC, NC           'select Next Channel
 X = INP(ADC)          'get current channel data
 V = X MOD 16 + (INP(ADC + 1) MOD 16) * 10 + (INP(ADC + 2) MOD 16) * 100
 V = V + (INP(ADC + 3) MOD 16) * 1000   'convert current channel data
 OUT START, 0                           'start conversion for Next Channel
 IF X AND 32 THEN V = V + 10000
 IF X AND 64 THEN V = -V
 V = V * .25                     'scale in millivolts
RETURN    '     READCARD

               'FRACDAY, calculate fraction of day, check for change of day
FRACDAY:
 SCANTIME$ = TIME$
 FOD = TIMER / 86400!     'FOD = Fraction Of Day
 IF LASTDAY$ <> DATE$ THEN DAY = DAY + 1: LASTDAY$ = DATE$
RETURN   '      FRACDAY

'               ERROROUTINE, error handler for all error trapping
ERROROUTINE:
'if printer in compressed mode and exiting the program then QUIT = 1
   IF (PRTCOMPRESS% AND (ERRORSTR$ = EXITERROR$)) THEN QUIT = 1 ELSE QUIT = 0
   CLS : LOCATE 10, 10: PRINT ERRORSTR$           'Error string
   LOCATE 11, 10: PRINT "BASIC error code:   "; ERR     'BASIC error code
   LOCATE 13, 10
   IF QUIT = 1 THEN PRINT "Press Esc to return to DOS";
   IF QUIT = 0 THEN PRINT "Press Esc to return to the Main Menu";
   A$ = INPUT$(1)
   IF A$ <> CHR$(27) THEN GOTO ERROROUTINE  'loop till valid keystroke
   RESUME CLOSEFILES       'Esc pressed then close files & go to Main Menu
                           'or exit the program
CLOSEFILES:
 ERRORSTR$ = "An error has occurred while closing a device file "
 CLOSE                      'close all devices
                            'exit program or return to the Main Menu
 IF QUIT THEN GOTO EXITPROGRAM ELSE GOTO MAINMENU

